<?php
// $Id: publicbookings.admin.inc,v 1.4 2008/07/08 09:07:43 carson Exp $
/**
 * @file
 * Administrative forms and callbacks.
 *
 */

/**
 * Bookings overview page.
 */
function publicbookings_overview_page() {
  $text = '<h3>'.t('Awaiting approval').'</h3>';
  $text .= publicbookings_booking_list('status=1');
  $text .= '<h3>'.t('Finalized').'</h3>';
  $text .= publicbookings_booking_list('status=2',1);
  return $text;
}

function publicbookings_booking_list($where = '', $element = 0) {
  $page_size = 15;
  if(!empty($where)) $where = 'WHERE ' . $where;
  $header = array(
    'Resource',
    array('data'=>'Name','field'=>'name'),
    'Client',
    array('data'=>'Start','field'=>'start'),
    'End',
    '',
  );
  $sql = "SELECT {bookings_records}.*, {bookings_resources}.name AS resname, {publicbookings_clients}.first_name,
         {publicbookings_clients}.last_name FROM {bookings_records}
    LEFT JOIN {bookings_resources} ON {bookings_resources}.resource_id={bookings_records}.resource_id
    LEFT JOIN {publicbookings_clients} ON {bookings_records}.record_id={publicbookings_clients}.record_id
    $where " . tablesort_sql($header);
  $q = pager_query(db_rewrite_sql($sql,'bookings_records','record_id'), $page_size, $element);
  $list = array();
  while($row = db_fetch_array($q)) {
    $list[] = $row;
  }
  // TODO: split off into theme function
  $path = '/admin/content/publicbookings/booking/';
  $data = array();
  foreach($list as $k=>$rec) {
    $edit = "<a href=\"$path{$rec['record_id']}/edit\">edit</a> ";
    $del = "<a href=\"$path{$rec['record_id']}/delete\">delete</a> ";
    $actions = $edit . $del;
    $client_name = empty($rec['first_name']) ? 'N/A' : $rec['last_name'].', '.$rec['first_name'];
    $data[] = array(
      $rec['resname'],
      $rec['name'],
      $client_name,
      $rec['start'], $rec['end'],
      $actions,
    );
  }
  return theme_table($header,$data) . theme('pager', null, $page_size);
}

/**
 * Bookings add page.
 */
function publicbookings_booking_add() {
  return drupal_get_form('publicbookings_booking_add_form');
}

function publicbookings_booking_edit($booking) {
  return drupal_get_form('publicbookings_booking_edit_form', $booking);
}

function publicbookings_booking_form(&$form_state, $booking = array()) {
  include_once('./'. drupal_get_path('module', 'bookingsapi') .'/bookingsapi.forms.inc');
  $form = bookingsapi_booking_form($booking);
  if (empty($booking['start']) && empty($booking['end'])) {
    $default_date = date_format(date_now(), DATE_DATETIME);
    $form['basic']['start']['#default_value'] = $default_date;
    $form['basic']['end']['#default_value'] = $default_date;
  }
  $form['#base'] = 'publicbookings_booking_form';
  $form['#submit'] = array('publicbookings_booking_form_submit');
  $form['#validate'][] = 'publicbookings_booking_form_validate';
  return $form;
}

function publicbookings_booking_form_validate($form, &$form_state) {
  if(!empty($form_state['values']['ical_rrule'])) {
    $form_state['values']['ical_rrule_until'] = bookingsapi_rrule_until_as_dt($form_state['values']['ical_rrule']);
    if(empty($form_state['values']['ical_rrule_until'])) {
      form_set_error('ical_rrule', t('Recurrences must have an end date.'));
    }
  }
  $fields = '{bookings_records}.name, {bookings_schedules}.type, {bookings_schedules}.`start`, {bookings_schedules}.`end`';
  $conflicts = bookingsapi_conflict_check($form_state['values'], $fields, 10);
  if (count($conflicts) > 0) {
    // TODO: create theme function
    // get resource name
    $errormsg = '<strong>'.t('Conflicts found').'</strong>'.
        '<p>'.t('The following conflicts with your requested booking:').'</p><ul>';
    foreach ($conflicts as $c) {
      $errormsg .= '<li>'.publicbookings_types_translate($c['type']).': '.$c['name'].', from '.$c['start'].' to '.$c['end'].'</li>';
    }
    $errormsg .= '</ul>';
    form_set_error('basic', $errormsg);
  }
}

function publicbookings_booking_form_submit($form, &$form_state) {
  $save_result = bookingsapi_booking_save($form_state['values']);
  $t_args = array('%name' => $form_state['values']['name']);
  // no message for BOOKINGSAPI_TIME_CONFLICT, as we should have prevented it before we got here
  switch ($save_result) {
    case SAVED_NEW:
      $op = 'created'; break;
    case SAVED_UPDATED:
      $op = 'updated'; break;
    default:
      $op = 'failed';
  }
  drupal_set_message(t("The booking %name has been $op.", $t_args));
  $form_state['redirect'] = 'admin/content/publicbookings/booking/list';
}

function publicbookings_resource_overview() {
  return publicbookings_resource_list();
}

/**
 * Resource listing page.
 */
function publicbookings_resource_list($where = '', $element = 0) {
  $page_size = 15;
  $header = array(
    array('data'=>'Name','field'=>'name'),
    'Location',
    'Parent',
    array('data'=>'Enabled','field'=>'disabled'),
    '',
  );
  $sql = "SELECT {bookings_resources}.*, parent.name AS parentname FROM {bookings_resources}
    LEFT JOIN {bookings_resources} AS parent ON parent.resource_id={bookings_resources}.parent
    $where " . tablesort_sql($header);
  $q = pager_query(db_rewrite_sql($sql,'bookings_resources','resource_id'), $page_size, $element);
  $list = array();
  while($row = db_fetch_array($q)) {
    $list[] = $row;
  }
  // TODO: split off into theme function
  $path = '/admin/content/publicbookings/resource/';
  $data = array();
  foreach($list as $k=>$res) {
    $enabled = $res['disabled'] ? 'No' : 'Yes';
    $edit = "<a href=\"$path{$res['resource_id']}/edit\">edit</a> ";
    $del = "<a href=\"$path{$res['resource_id']}/delete\">delete</a> ";
    $avail = "<a href=\"$path{$res['resource_id']}/availability\">availability</a>";
    $actions = $edit . $del . $avail;
    $parent = empty($res['parentname']) ? '-' : $res['parentname'];
    $data[] = array($res['name'], $res['location'], $parent, $enabled, $actions);
  }
  return theme_table($header,$data) . theme('pager', null, $page_size);
}

/**
 * Resource adding page.
 */
function publicbookings_resource_add() {
  return drupal_get_form('publicbookings_resource_add_form');
}

/**
 * Resource editing page.
 */
function publicbookings_resource_edit($resource) {
  return drupal_get_form('publicbookings_resource_edit_form', $resource);
}

/**
 * Resource delete confirmation page.
 * @param array $resource
 */
function publicbookings_resource_delete($resource) {
  return drupal_get_form('publicbookings_resource_delete_confirm', $resource);
}

/**
 * Resource delete confirmation form.
 */
function publicbookings_resource_delete_confirm(&$form_state, $resource) {
  $form['#item'] = $resource;
  $desc = t('All booking and availability records associated with %name will also be deleted.', array('%name'=>$resource['name']));
  return confirm_form($form, t('Are you sure you want to delete this resource?'),
    'admin/content/publicbookings/resource',$desc,t('Delete'));
}

/**
 * Resource delete.
 */
function publicbookings_resource_delete_confirm_submit($form, &$form_state) {
  bookingsapi_resource_delete($form['#item']);
  $t_args = array('%name' => $form['#item']['name']);
  drupal_set_message(t('The bookable resource %name has been deleted.', $t_args));
  $form_state['redirect'] = 'admin/content/publicbookings/resource';
}

/**
 * Resources form.
 */
function publicbookings_resource_form(&$form_state, $resource = array()) {
  include_once('./'. drupal_get_path('module', 'bookingsapi') .'/bookingsapi.forms.inc');
  $form = bookingsapi_resource_form($resource);
  if(!empty($resource)) $form['delete']['#submit'] = array('publicbookings_resource_delete');
  $form['#submit'] = array('publicbookings_resource_form_submit');
  $form['#validate'] = array('publicbookings_resource_form_validate');
  return $form;
}

/**
 * Resource form validation.
 */
function publicbookings_resource_form_validate($form, &$form_state) {

}

/**
 * Resource form submit.
 */
function publicbookings_resource_form_submit($form, &$form_state) {
  $save_result = bookingsapi_resource_save($form_state['values']);
  $t_args = array('%name' => $form_state['values']['name']);
  switch($save_result) {
    case SAVED_NEW:
      $op = 'created'; break;
    case SAVED_UPDATED:
      $op = 'updated'; break;
    default:
      $op = 'failed';
  }
  drupal_set_message(t("The bookable resource %name has been $op.", $t_args));
  if($op=='failed') return false;
  $form_state['redirect'] = 'admin/content/publicbookings/resource';
}


/**
 * Availability list page.
 */
function publicbookings_availability_list($resource) {
  include_once('./'. drupal_get_path('module', 'bookingsapi') .'/bookingsapi.forms.inc');
  $avail_code = bookingsapi_availability_types($resource['default_availability']);
  $text .= '<h3>'.$resource['name'].'</h3>';
  $text .= '<strong>'.t('Default availability:').'</strong> ' .
    bookingsapi_availability_types($avail_code);
  return $text;
}

/**
 * Availability add page.
 */
function publicbookings_availability_add($resource) {
  include_once('./'. drupal_get_path('module', 'bookingsapi') .'/bookingsapi.forms.inc');
  $fakeavail = array('resource_id'=>$resource['resource_id']);
  return drupal_get_form('publicbookings_availability_add_form', $fakeavail);
}

function publicbookings_availability_form(&$form_state, $avail = array()) {
  include_once('./'. drupal_get_path('module', 'bookingsapi') .'/bookingsapi.forms.inc');
  $form = bookingsapi_availability_form($avail);
  if (empty($avail['start']) && empty($avail['end'])) {
    $default_date = date_format(date_now(), DATE_DATETIME);
    $form['basic']['start']['#default_value'] = $default_date;
    $form['basic']['end']['#default_value'] = $default_date;
  }
  if ($avail['resource_id'] > 0) {
    // if a resource ID is given, don't let the user change it
    $form['basic']['resource_id']['#value'] = $avail['resource_id'];
    $form['basic']['resource_id']['#disabled'] = TRUE;
    $res = bookingsapi_resource_load($avail['resource_id']);
    if($res['disabled']) {
      // disabled resources don't appear in the dropdown menu be default
      // so if disabled, make the menu show all
      $form['basic']['resource_id']['#bookingsapi_showall'] = TRUE;
    }
  }
  $form['#base'] = 'publicbookings_availability_form';
  $form['#submit'] = array('publicbookings_availability_form_submit');
  $form['#validate'][] = 'publicbookingsavailability_form_validate';
  return $form;
}

// temporarily identical to booking equivalent
function publicbookings_availability_form_validate($form, &$form_state) {
if(!empty($form_state['values']['ical_rrule'])) {
    $form_state['values']['ical_rrule_until'] = bookingsapi_rrule_until_as_dt($form_state['values']['ical_rrule']);
    if(empty($form_state['values']['ical_rrule_until'])) {
      form_set_error('ical_rrule', t('Recurrences must have an end date.'));
    }
  }
  $fields = '{bookings_records}.name, {bookings_schedules}.type, {bookings_schedules}.`start`, {bookings_schedules}.`end`';
  $conflicts = bookingsapi_conflict_check($form_state['values'], $fields, 10);
  if (count($conflicts) > 0) {
    // TODO: create theme function
    // get resource name
    $errormsg = '<strong>'.t('Conflicts found').'</strong>'.
        '<p>'.t('The following conflicts with your requested booking:').'</p><ul>';
    foreach ($conflicts as $c) {
      $errormsg .= '<li>'.publicbookings_types_translate($c['type']).': '.$c['name'].', from '.$c['start'].' to '.$c['end'].'</li>';
    }
    $errormsg .= '</ul>';
    form_set_error('basic', $errormsg);
  }
}

// temporarily identical to booking equivalent
function publicbookings_availability_form_submit($form, &$form_state) {
  $save_result = bookingsapi_availability_save($form_state['values']);
  $t_args = array('%name' => $form_state['values']['name']);
  // no message for BOOKINGSAPI_TIME_CONFLICT, as we should have prevented it before we got here
  switch ($save_result) {
    case SAVED_NEW:
      $op = 'created'; break;
    case SAVED_UPDATED:
      $op = 'updated'; break;
    default:
      $op = 'failed';
  }
  drupal_set_message(t("The availability %name has been $op.", $t_args));
  $form_state['redirect'] = 'admin/content/publicbookings/resource/'.$form_state['values']['resource_id'].'/availability';
}

/**
 * Settings page.
 */
function publicbookings_settings() {
  return drupal_get_form('publicbookings_settings_form');
}

/**
 * Settings form.
 */
function publicbookings_settings_form() {
  $form = array();
  $form['basic'] = array(
    '#type' => 'fieldset',
    '#title' => 'Basic settings'
  );
  $form['basic']['publicbookings_enabled'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable the public bookings form'),
    '#default_value' => variable_get('publicbookings_enabled', 0),
  );

  $form['#validate'] = array(
    'publicbookings_settings_validate' => array()
  );
  return system_settings_form($form);
}

/**
 * Settings form validation.
 */
function publicbookings_settings_validate($form, &$form_state) {

}

/**
 * Settings form submit.
 */
function publicbookings_settings_submit($form, &$form_state) {
  variable_set('publicbookings_enabled', $form_state['values']['publicbookings_enabled']);
}