<?php
// $Id: publicbookings.public.inc,v 1.1 2008/07/08 07:44:39 carson Exp $
/**
 * @file
 * Public Bookings - Public interface
 */

function publicbookings_clientinfo_form($values) {
  $form = array();
  $form['client_id'] = array(
    '#type' => 'value',
    '#value' => $values['client_id'],
  );
  $form['client'] = array(
    '#title' => t('Your details'),
    '#type' => 'fieldset',
  );
  $form['client']['first_name'] = array(
    '#title' => t('First name'),
    '#type' => 'textfield',
    '#default_value' => $values['first_name'],
  );
  $form['client']['last_name'] = array(
    '#title' => t('Last name'),
    '#type' => 'textfield',
    '#default_value' => $values['last_name'],
  );
  $form['client']['phone'] = array(
    '#title' => t('Phone number'),
    '#type' => 'textfield',
    '#description' => t('10 digits.'),
    '#default_value' => $values['phone'],
  );
  $form['client']['email'] = array(
    '#title' => t('Email address'),
    '#type' => 'textfield',
    '#description' => t('Your booking will be confirmed by email.'),
    '#default_value' => $values['email'],
  );
  $form['client']['comments'] = array(
    '#title' => t('Special instructions or comments'),
    '#type' => 'textarea',
    '#description' => t('If this booking has any special requirements, specify them here.'),
    '#default_value' => $values['comments'],
  );
  $form['#validate'] = array('publicbookings_clientinfo_form_validate');

  return $form;
}

function publicbookings_clientinfo_form_validate($form, &$form_state) {
  // remove all non-digits
  $form_state['values']['phone'] = preg_replace('/\D/','',$form_state['values']['phone']);
}

function publicbookings_passphrase_form() {
  $form = array();
  $form['client'] = array(
    '#title' => t('Update request'),
    '#type' => 'fieldset',
  );
  $form['client']['passphrase'] = array(
    '#title' => t('Passphrase'),
    '#type' => 'textfield',
    '#description' => t('Your booking request\'s passphrase is found in your initial confirmation email.'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Proceed'),
  );
  $form['#validate'] = array('publicbookings_passphrase_validate');
  $form['#submit'] = array('publicbookings_passphrase_submit');
  return $form;
}

function publicbookings_request_form(&$form_state, $values) {
  include_once('./'. drupal_get_path('module', 'bookingsapi') .'/bookingsapi.forms.inc');
  $form = array_merge(bookingsapi_booking_form($values), publicbookings_clientinfo_form($values));
  $form['basic']['#title'] = t('Booking request');
  // status is always 'open' for a booking request
  $form['basic']['status'] = array(
    '#type' => 'value',
    '#value' => 1,
  );
  if (empty($booking['start']) && empty($booking['end'])) {
    $default_date = date_format(date_now(), DATE_DATETIME);
    $form['basic']['start']['#default_value'] = $default_date;
    $form['basic']['end']['#default_value'] = $default_date;
  }
  // repeat functionality is probably unnecessary for public use
  unset($form['basic']['ical_rrule']);
  // implement later
  unset($form['basic']['priority']);
  $form['submit']['#weight'] = 8;
  $form['submit']['#value'] = t('Submit request');
  $form['#validate'][] = 'publicbookings_request_form_validate';
  $form['#submit'] = array('publicbookings_request_form_submit');
  return $form;
}

function publicbookings_public_booking_form_validate($form, &$form_state) {
  $fields = '{bookings_records}.name, {bookings_schedules}.type, {bookings_schedules}.`start`, {bookings_schedules}.`end`';
  $conflicts = bookingsapi_conflict_check($form_state['values'], $fields, 10);
  if(count($conflicts) > 0) {
    // TODO: create theme function
    // get resource name
    $errormsg = '<strong>'.t('Conflicts found').'</strong>'.
        '<p>'.t('The following conflicts with your requested booking:').'</p><ul>';
    foreach($conflicts as $c) {
      $errormsg .= '<li>'.publicbookings_types_translate($c['type']).': '.$c['name'].', from '.$c['start'].' to '.$c['end'].'</li>';
    }
    $errormsg .= '</ul>';
    form_set_error('basic', $errormsg);
  }
}

function publicbookings_request_form_submit($form, &$form_state) {
  $save_result = publicbookings_public_request_save($form_state['values']);
  $t_args = array('%name' => $form_state['values']['name']);
  $blurb = '';
  // no message for BOOKINGSAPI_TIME_CONFLICT, as we should have prevented it before we got here
  switch ($save_result) {
    case SAVED_NEW:
      $op = 'created';
      $blurb .= '<p>'.t('Your passphrase is ').'<strong>'.$form_state['values']['passphrase'].'</strong>. '.
            t('Write it down; you can use this passphrase to return to your booking request and make changes
            if necessary').'.</p>';
      break;
    case SAVED_UPDATED:
      $op = 'updated';
      sess_write('publicbookings_authed_req_id', NULL);
      break;
  }
  $blurb = '<p>'.t("Your booking request %name has been $op.", $t_args).'</p>' . $blurb;
  //$form_state['redirect'] = 'admin/content/publicbookings/booking/list';
  drupal_set_message($blurb);
}

function publicbookings_public_overview_page() {
  $text = '<h3>'.t('Currently occupied').'</h3>';
  $text .= '<p>Not yet implemented.</p>';
  $text .= '<h3>'.t('Upcoming').'</h3>';
  $text .= '<p>Not yet implemented.</p>';
  return $text;
}

function publicbookings_public_request_add() {
  if(variable_get('publicbookings_enabled', 0) == 0) {
    return publicbookings_public_disabled_message();
  }
  return drupal_get_form('publicbookings_request_form', array());
}

function publicbookings_public_request_save(&$values) {
  $record_save = bookingsapi_booking_save($values);
  if($record_save) {
    if(empty($values['passphrase'])) $values['passphrase'] = publicbookings_generate_passphrase(12);
    $update = empty($values['client_id']) ? array() : 'client_id';
    drupal_write_record('publicbookings_clients', &$values, $update);
    $values = (array) $values;
  }
  return $record_save;
}

function publicbookings_generate_passphrase($length) {
  $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
  $pass = '';
  while($length > 0) {
    $pass .= substr($chars, rand(0,35), 1);
    $length--;
  }
  return $pass;
}

function publicbookings_public_passphrase() {
  if(variable_get('publicbookings_enabled', 0) == 0) {
    return publicbookings_public_disabled_message();
  }
  $blurb = '<p>'.t('To update or cancel your booking request, enter the unique passphrase associated with your booking.').'</p>'.
           '<p>'.t('If you have lost your passphrase, or if your booking status has already been set to finalized, contact
           an administrator to make changes.').'</p>';
  return $blurb . drupal_get_form('publicbookings_passphrase_form');
}

function publicbookings_passphrase_validate($form, &$form_state) {
  $sql = "SELECT * FROM {publicbookings_clients} WHERE passphrase='%s'";
  $q = db_query($sql, $form_state['values']['passphrase']);
  $client = db_fetch_array($q);
  if($client == FALSE) {
    form_set_error('passphrase', t('Invalid passphrase.'));
  } else {
    $form_state['client'] = $client;
  }
}

function publicbookings_passphrase_submit($form, &$form_state) {
  session_save_session(TRUE);
  sess_write('publicbookings_authed_req_id', $form_state['client']['client_id']);
  $form_state['redirect'] = 'publicbookings/request/'.$form_state['client']['client_id'].'/update';
}

function publicbookings_public_request_update($client) {
  if(sess_read('publicbookings_authed_req_id') != $client['client_id']) {
    return drupal_access_denied();
  }
  $booking = bookingsapi_booking_load($client['record_id']);
  $values = array_merge($booking, $client);
  var_dump($values);
  return drupal_get_form('publicbookings_request_form', $values);
}

function publicbookings_public_disabled_message() {
  $blurb = t('The public booking system is currently disabled. Try again later.');
  $blurb = '<p>'.$blurb.'</p>';
  return $blurb;
}